﻿Set-ExecutionPolicy -Scope Process -ExecutionPolicy Bypass

Add-Type -assembly System.Windows.Forms
$main_form = New-Object System.Windows.Forms.Form
$main_form.Text ='IPVFC Updater'
$main_form.Width = 600
$main_form.Height = 400
$main_form.AutoSize = $true

function Show-MessageBox {  
    [CmdletBinding()]  
    Param (   
        [Parameter(Mandatory = $false)]  
        [string]$Title = 'Warning!',

        [Parameter(Mandatory = $true)]
        [string]$Message,  

        [Parameter(Mandatory = $false)]
        [ValidateSet('OK', 'OKCancel', 'AbortRetryIgnore', 'YesNoCancel', 'YesNo', 'RetryCancel')]
        [string]$Buttons = 'OKCancel',

        [Parameter(Mandatory = $false)]
        [ValidateSet('Error', 'Warning', 'Information', 'None', 'Question')]
        [string]$Icon = 'Information',

        [Parameter(Mandatory = $false)]
        [ValidateRange(1,3)]
        [int]$DefaultButton = 1
    )            

    # determine the possible default button
    if ($Buttons -eq 'OK') {
        $Default = 'Button1'
    }
    elseif (@('AbortRetryIgnore', 'YesNoCancel') -contains $Buttons) {
        $Default = 'Button{0}' -f [math]::Max([math]::Min($DefaultButton, 3), 1)
    }
    else {
        $Default = 'Button{0}' -f [math]::Max([math]::Min($DefaultButton, 2), 1)
    }

    Add-Type -AssemblyName System.Windows.Forms
    # added from tip by [Ste](https://stackoverflow.com/users/8262102/ste) so the 
    # button gets highlighted when the mouse hovers over it.
    [void][System.Windows.Forms.Application]::EnableVisualStyles()

    # Setting the first parameter 'owner' to $null lets he messagebox become topmost
    [System.Windows.Forms.MessageBox]::Show($null, $Message, $Title,   
                                            [Windows.Forms.MessageBoxButtons]::$Buttons,   
                                            [Windows.Forms.MessageBoxIcon]::$Icon,
                                            [Windows.Forms.MessageBoxDefaultButton]::$Default)
}

$Label = New-Object System.Windows.Forms.Label
$Label.Text = "Please ensure that d3Service and d3Buddy have both been stopped before proceeding!"
$Label.Location  = New-Object System.Drawing.Point(10,10)
$Label.AutoSize = $true
$main_form.Controls.Add($Label)

$Button = New-Object System.Windows.Forms.Button
$Button.Location = New-Object System.Drawing.Size(10,40)
$Button.Size = New-Object System.Drawing.Size(220,30)
$Button.Text = "Kill d3Service, d3Buddy"
$Button.Add_Click(
    {
        # Kill d3Buddy then d3Service
        $d3buddy = Get-Process d3Buddy -ErrorAction SilentlyContinue
        if ($d3buddy)
        {
            # try gracefully first
            $d3buddy.CloseMainWindow()
            # kill after five seconds
            Sleep 2
            if (!$d3buddy.HasExited)
            {
                $d3buddy | Stop-Process -Force
            }
        }

        $d3service = Get-Process d3Service -ErrorAction SilentlyContinue
        if ($d3service)
        {
            # try gracefully first
            $d3service.CloseMainWindow()
            # kill after five seconds
            Sleep 2
            if (!$d3service.HasExited)
            {
                $d3service | Stop-Process -Force
            }
        }
    }
)
$main_form.Controls.Add($Button)

$LogString = "$(".\ipvfc_updater.exe log -b 115200")"
$logOutput = (Invoke-Expression -Command $LogString)
$slotStatArray = $logOutput.Split("\n")

$MyGroupBox = New-Object System.Windows.Forms.GroupBox
$MyGroupBox.Location = '10,200'
$MyGroupBox.size = '300,150'
$MyGroupBox.text = "Select VFC Slots"

$slot1 = New-Object System.Windows.Forms.CheckBox
$slot1.Location = '10,30'
$slot1.size = '260,20'
$slot1.Checked = $false
$slot1.Enabled = $false
$slot1.Text = "1"
foreach ($item in $slotStatArray)
{
    if ($item[0] -eq "1")
    {
        $slot1.Text = $item
        $slot1.Enabled = $true
    }
}

$slot2 = New-Object System.Windows.Forms.CheckBox
$slot2.Location = '10,60'
$slot2.size = '260,20'
$slot2.Checked = $false
$slot2.Enabled = $false
$slot2.Text = "2"
foreach ($item in $slotStatArray)
{
    if ($item[0] -eq "2")
    {
        $slot2.Text = $item
        $slot2.Enabled = $true
    }
}

$slot3 = New-Object System.Windows.Forms.CheckBox
$slot3.Location = '10,90'
$slot3.size = '260,20'
$slot3.Checked = $false
$slot3.Enabled = $false
$slot3.Text = "3"
foreach ($item in $slotStatArray)
{
    if ($item[0] -eq "3")
    {
        $slot3.Text = $item
        $slot3.Enabled = $true
    }
}

$slot4 = New-Object System.Windows.Forms.CheckBox
$slot4.Location = '10,120'
$slot4.size = '260,20'
$slot4.Checked = $false
$slot4.Enabled = $false
$slot4.Text = "4"
foreach ($item in $slotStatArray)
{
    if ($item[0] -eq "4")
    {
        $slot4.Text = $item
        $slot4.Enabled = $true
    }
}

$MyGroupBox.Controls.AddRange(@($slot1, $slot2, $slot3, $slot4))
$main_form.Controls.Add($MyGroupBox)

$BitFilePath = ""
$FlashbinFilePath = ""

$BitLabel = New-Object System.Windows.Forms.Label
$BitLabel.Text = $BitFilePath
$BitLabel.Location  = New-Object System.Drawing.Point(240,125)
$BitLabel.AutoSize = $true
$main_form.Controls.Add($BitLabel)

$FlashbinLabel = New-Object System.Windows.Forms.Label
$FlashbinLabel.Text = $FlashbinFilePath
$FlashbinLabel.Location  = New-Object System.Drawing.Point(240,165)
$FlashbinLabel.AutoSize = $true
$main_form.Controls.Add($FlashbinLabel)

$BitButton = New-Object System.Windows.Forms.Button
$BitButton.Location = New-Object System.Drawing.Size(10,120)
$BitButton.Size = New-Object System.Drawing.Size(220,30)
$BitButton.Text = "Select bit file"
$BitButton.Add_Click(
{
    $initialDirectory = [Environment]::GetFolderPath('Desktop')
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    $OpenFileDialog.InitialDirectory = $initialDirectory
    $OpenFileDialog.Filter = 'bit files (*.bit)|*.bit'
    $OpenFileDialog.Multiselect = $false
    $response = $OpenFileDialog.ShowDialog( ) # $response can return OK or Cancel
    if ( $response -eq 'OK' )
    {
        $BitFilePath = $OpenFileDialog.FileName
        $BitLabel.Text = $OpenFileDialog.FileName
    }
}
)
$main_form.Controls.Add($BitButton)

$BinButton = New-Object System.Windows.Forms.Button
$BinButton.Location = New-Object System.Drawing.Size(10,160)
$BinButton.Size = New-Object System.Drawing.Size(220,30)
$BinButton.Text = "Select flashbin file"
$BinButton.Add_Click(
{
    $initialDirectory = [Environment]::GetFolderPath('Desktop')
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    $OpenFileDialog.InitialDirectory = $initialDirectory
    $OpenFileDialog.Filter = 'flashbin files (*.flashbin)|*.flashbin'
    $OpenFileDialog.Multiselect = $false
    $response = $OpenFileDialog.ShowDialog( ) # $response can return OK or Cancel
    if ( $response -eq 'OK' )
    {
        $FlashbinFilePath = $OpenFileDialog.FileName
        $FlashbinLabel.Text = $OpenFileDialog.FileName
    }
}
)
$main_form.Controls.Add($BinButton)

$textBox = New-Object System.Windows.Forms.RichTextBox
$textBox.Location = New-Object System.Drawing.Size(10,370)
$textBox.Size = New-Object System.Drawing.Size(700,100)
$textBox.ReadOnly = $true
$textBox.AcceptsTab = $false
$textBox.Multiline = $true
$textBox.ScrollBars = 'Both'
$textBox.Text = ""
$main_form.Controls.Add($textBox)


$RedetectButton = New-Object System.Windows.Forms.Button
$RedetectButton.Location = New-Object System.Drawing.Size(320,240)
$RedetectButton.Size = New-Object System.Drawing.Size(220,30)
$RedetectButton.Text = "Redetect IPVFC Cards"
$RedetectButton.Add_Click(
{
    $LogString = "$(".\ipvfc_updater.exe log -b 115200")"
    $logOutput = (Invoke-Expression -Command $LogString)
    $slotStatArray = $logOutput.Split("\n")

    $slot1.Text = "1"
    $slot1.Checked = $false
    $slot1.Enabled = $false

    $slot2.Text = "2"
    $slot2.Checked = $false
    $slot2.Enabled = $false

    $slot3.Text = "3"
    $slot3.Checked = $false
    $slot3.Enabled = $false

    $slot4.Text = "4"
    $slot4.Checked = $false
    $slot4.Enabled = $false

    foreach ($item in $slotStatArray)
    {
        if ($item[0] -eq "1")
        {
            $slot1.Text = $item
            $slot1.Enabled = $true
        }
        if ($item[0] -eq "2")
        {
            $slot2.Text = $item
            $slot2.Enabled = $true
        }
        if ($item[0] -eq "3")
        {
            $slot3.Text = $item
            $slot3.Enabled = $true
        }
        if ($item[0] -eq "4")
        {
            $slot4.Text = $item
            $slot4.Enabled = $true
        }
    }
    $main_form.Update()
}
)
$main_form.Controls.Add($RedetectButton)



$UpdateButton = New-Object System.Windows.Forms.Button
$UpdateButton.Location = New-Object System.Drawing.Size(320,280)
$UpdateButton.Size = New-Object System.Drawing.Size(220,30)
$UpdateButton.Text = "Update"
$UpdateButton.Add_Click(
{
    # First check if update files have been provided
    if ([string]::IsNullOrWhitespace($FlashbinLabel.Text) -or [string]::IsNullOrWhitespace($BitLabel.Text))
    {
        Show-MessageBox -Title 'Warning!' -Message 'Please provide a valid bit and flashbin file' -Icon Information -Buttons OK

        return
    }

    if ($slot1.Checked)
    {
        $slotSelection = "-s 1"
        $prettySlotSelection = "slot_1"
        $SWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t SW -w STD -f") `"$($FlashbinLabel.Text)`""
        $FWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t FPGA -w STD -r -f") `"$($BitLabel.Text)`""
        $logFilename = ".\update_log " + $prettySlotSelection + ".txt"
        Write-Host $SWUpdateString
        Write-Host $FWUpdateString
        Write-Host $logFilename

        $command = {
            param($swstring,$fwstring,$logstring)
            Invoke-Expression -Command $swstring | Tee-Object -file $logstring | Write-Host
            Invoke-Expression -Command $fwstring | Tee-Object -file $logstring -Append | Write-Host
        }       
        Start-Process powershell -ArgumentList "-noexit -command (Invoke-Command -ScriptBlock {$command} -ArgumentList '$SWUpdateString', '$FWUpdateString', '$logFilename')"
        Start-Sleep -Seconds 10
    }
    if ($slot2.Checked)
    {
        $slotSelection = "-s 2"
        $prettySlotSelection = "slot_2"
        $SWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t SW -w STD -f") `"$($FlashbinLabel.Text)`""
        $FWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t FPGA -w STD -r -f") `"$($BitLabel.Text)`""
        $logFilename = ".\update_log " + $prettySlotSelection + ".txt"
        Write-Host $SWUpdateString
        Write-Host $FWUpdateString
        Write-Host $logFilename

        $command = {
            param($swstring,$fwstring,$logstring)
            Invoke-Expression -Command $swstring | Tee-Object -file $logstring | Write-Host
            Invoke-Expression -Command $fwstring | Tee-Object -file $logstring -Append | Write-Host
        }       
        Start-Process powershell -ArgumentList "-noexit -command (Invoke-Command -ScriptBlock {$command} -ArgumentList '$SWUpdateString', '$FWUpdateString', '$logFilename')"
        Start-Sleep -Seconds 10   
    }
    if ($slot3.Checked)
    {
        $slotSelection = "-s 3"
        $prettySlotSelection = "slot_3"
        $SWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t SW -w STD -f") `"$($FlashbinLabel.Text)`""
        $FWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t FPGA -w STD -r -f") `"$($BitLabel.Text)`""
        $logFilename = ".\update_log " + $prettySlotSelection + ".txt"
        Write-Host $SWUpdateString
        Write-Host $FWUpdateString
        Write-Host $logFilename

        $command = {
            param($swstring,$fwstring,$logstring)
            Invoke-Expression -Command $swstring | Tee-Object -file $logstring | Write-Host
            Invoke-Expression -Command $fwstring | Tee-Object -file $logstring -Append | Write-Host
        }       
        Start-Process powershell -ArgumentList "-noexit -command (Invoke-Command -ScriptBlock {$command} -ArgumentList '$SWUpdateString', '$FWUpdateString', '$logFilename')"
        Start-Sleep -Seconds 10
    }
    if ($slot4.Checked)
    {
        $slotSelection = "-s 4"
        $prettySlotSelection = "slot_4"
        $SWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t SW -w STD -f") `"$($FlashbinLabel.Text)`""
        $FWUpdateString = "$(".\ipvfc_updater.exe update -b 115200") $($slotSelection) $("-t FPGA -w STD -r -f") `"$($BitLabel.Text)`""
        $logFilename = ".\update_log " + $prettySlotSelection + ".txt"
        Write-Host $SWUpdateString
        Write-Host $FWUpdateString
        Write-Host $logFilename

        $command = {
            param($swstring,$fwstring,$logstring)
            Invoke-Expression -Command $swstring | Tee-Object -file $logstring | Write-Host
            Invoke-Expression -Command $fwstring | Tee-Object -file $logstring -Append | Write-Host
        }       
        Start-Process powershell -ArgumentList "-noexit -command (Invoke-Command -ScriptBlock {$command} -ArgumentList '$SWUpdateString', '$FWUpdateString', '$logFilename')"

    }

    <#
    Invoke-Expression -Command $SWUpdateString | Tee-Object -file $logFilename | out-string -Stream | foreach-object {
    $textbox.lines = $textbox.lines + $_
    $textbox.Select($textbox.Text.Length, 0)
    $textbox.ScrollToCaret()
    $main_form.Update()}

    Invoke-Expression -Command $FWUpdateString | Tee-Object -file $logFilename -Append | out-string -Stream | foreach-object {
    $textbox.lines = $textbox.lines + $_
    $textbox.Select($textbox.Text.Length, 0)
    $textbox.ScrollToCaret()
    $main_form.Update()}
    #>
}
)
$main_form.Controls.Add($UpdateButton)

$main_form.ShowDialog()
